IF EXISTS ( SELECT *
              FROM SYSOBJECTS O, SYSUSERS U
             WHERE O.NAME = 'VERIFICA_ERRO_CTB'
               AND TYPE  = 'P'
               AND U.UID = O.UID
               AND U.NAME = USER )
DROP PROCEDURE VERIFICA_ERRO_CTB
GO

CREATE PROCEDURE VERIFICA_ERRO_CTB(
@P_NRLANCCTBTEMP VARCHAR(12),
@P_CDEMPRESA VARCHAR(2)

)

AS
BEGIN
   DECLARE @V_COUNT NUMERIC(13,2);
   DECLARE @V_RAT VARCHAR(01);
   DECLARE @V_CONTRAT VARCHAR(01);
   DECLARE @V_CONTCOME VARCHAR(01);
   DECLARE @C_CONTA_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_CODCOME_ID VARCHAR(01);
   DECLARE @C_CODCOME_CODIGO VARCHAR(21);
   DECLARE @C_CODCOME_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_PART_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_RAT_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_FIL_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_FIL_CDFILIAL VARCHAR(4);
   DECLARE @C_FIL_INF_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_CCUST_CDFILIAL VARCHAR(04);
   DECLARE @C_CCUST_CDCENTCUST VARCHAR(04);
   DECLARE @C_CCUST_NRSEQCTBTEMP VARCHAR (12);
   DECLARE @C_CCUST_INF_NRSEQCTBTEMP VARCHAR (12);
   DECLARE @C_DIF_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @C_DIF_VRLANCCTBTEMP NUMERIC(13,2);
   DECLARE @C_CODCOME_NULL_NRSEQCTBTEMP VARCHAR(12);
   DECLARE @V_IDSISTEMA VARCHAR(03);
   /*
   ---------    CDIGOS DE ERRO DE ACORDO COM A TELA CTB10200 ----------
   001 - Total de crdito difere do total de dbito.
   002 - Data fora do perodo contbil.
   003 - Partida %s - Conta contbil no cadastrada.
   004 - Partida %s - Histrico padro no cadastrado.
   005 - Partida %s - Cdigo comercial no cadastrado.
   006 - Partida %s - Valor da partida zerado.
   007 - Partida %s - Valor de rateio zerado.
   008 - Partida %s - Filial de rateio no cadastrada.
   009 - Partida %s - Centro de custo no cadastrado para filial informada.
   010 - Partida %s - Soma dos rateios difere do valor da partida.
   011 - Partida %s - Conta contbil sem cdigo comercial informado.
   012 - Partida %s - Conta Contbil Gerencial sem rateio informado.
   013 - Partida %s - Centro de custo deve ser informado.
   014 - -- RESERVADO PARA PRODUTO SEM CONTA CONTABIL --
   015 - Lanamento fora do perodo contbil para o financeiro. (BCO, PAG e REC)
   016 - Lanamento fora do perodo contbil de documentos fiscais de entrada. (DFE)
   017 - Lanamento fora do perodo contbil para documentos fiscais de sada. (DFS)
   */                                                                    


   SET @V_COUNT = 0;

   SELECT @V_CONTRAT = IDCONTGERE, @V_CONTCOME = IDCONTCOME
     FROM CONTOPECTBL
    WHERE CDEMPRESA = @P_CDEMPRESA;


   /*VERIFICAO DO TOTAL DE DBITO E CRDITO NO LANAMENTO CONTBIL */
   SELECT @V_COUNT = ISNULL(SUM(VRLANCCTBTEMP),0)
     FROM LANCCTBTEMP
    WHERE NRLANCCTBTEMP = @P_NRLANCCTBTEMP
      AND CDEMPRESA = @P_CDEMPRESA
   HAVING SUM(VRLANCCTBTEMP) <> 0;

   IF @V_COUNT <> 0 BEGIN
      INSERT INTO ERROCTBTEMP
         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
      VALUES
         (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '001');
      SET  @V_COUNT = 0;
   END;


   /*VERIFICA SE O LANCAMANTO CONTBIL EST DENRO DO PERODO CONTBIL */
   SELECT @V_COUNT = COUNT(*)
     FROM LANCCTBTEMP L, CONTOPECTBL C
    WHERE L.CDEMPRESA = @P_CDEMPRESA
      AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
      AND NOT L.DTLANCCTBTEMP BETWEEN C.DTINICCONT AND C.DTFINALANC
      AND L.CDEMPRESA = C.CDEMPRESA;

    IF @V_COUNT <> 0 BEGIN
       INSERT INTO ERROCTBTEMP
          (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
       VALUES
          (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '002');
    END;


    /* VERIFICA SE EXISTE ALGUMA CONTA CONTBIL QUE NO ESTEJA CADASTRTADA  */
    DECLARE C_CONTA CURSOR FOR
       (SELECT NRSEQCTBTEMP
          FROM LANCCTBTEMP L
         WHERE L.CDEMPRESA = @P_CDEMPRESA
           AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
           AND NOT EXISTS (SELECT 1
                             FROM CONTCTBL C
                            WHERE C.CDCONTCTBL = L.CDCONTCTBL));
       OPEN C_CONTA;
       FETCH NEXT FROM C_CONTA INTO @C_CONTA_NRSEQCTBTEMP;
       WHILE (@@FETCH_STATUS = 0) BEGIN
          INSERT INTO ERROCTBTEMP
             (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
          VALUES
              (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_CONTA_NRSEQCTBTEMP, '003');
          FETCH NEXT FROM C_CONTA INTO @C_CONTA_NRSEQCTBTEMP;
       END;
       CLOSE C_CONTA;
       DEALLOCATE C_CONTA;


   /*VERIFICA SE EXISTE ALGUM HISTRICO PADRO NO CADASTRADO*/
   SELECT @V_COUNT = COUNT(*)
     FROM LANCCTBTEMP L
    WHERE L.CDEMPRESA = @P_CDEMPRESA
      AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
      AND NOT EXISTS  (SELECT 1
                         FROM HISTPADR H
                        WHERE H.CDHISTPADR = L.CDHISTPADR);

   IF @V_COUNT <> 0 BEGIN
      INSERT INTO ERROCTBTEMP
         (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
      VALUES
         (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '004');
   END;

   IF @V_CONTCOME = 'S' BEGIN
      /*VERIFICAO DE EXISTE ALGUM CDIGO COMERCIAL QUE NO ESTEJA CADASTRADO*/
      DECLARE C_CODCOME CURSOR FOR
         (SELECT IDLANCCTBCOME ID, CDLANCCTBCOME CODIGO, NRSEQCTBTEMP
            FROM LANCCTBTEMP
           WHERE CDEMPRESA = @P_CDEMPRESA
             AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
             AND CDLANCCTBCOME IS NOT NULL
             AND IDLANCCTBCOME <> 'N');

         OPEN C_CODCOME;
         FETCH NEXT FROM C_CODCOME INTO @C_CODCOME_ID,@C_CODCOME_CODIGO, @C_CODCOME_NRSEQCTBTEMP;
         WHILE (@@FETCH_STATUS = 0) BEGIN

            IF @C_CODCOME_ID = 'B' BEGIN /*BANCO*/
               SELECT @V_COUNT = COUNT(*)
                 FROM CONTCORR C
                WHERE C.CDREDUCONT = @C_CODCOME_CODIGO;
            END
            ELSE IF @C_CODCOME_ID = 'F' BEGIN /*FORNECEDOR*/
               SELECT @V_COUNT = COUNT(*)
                 FROM FORNECEDOR
                WHERE CDFORNECED = @C_CODCOME_CODIGO;
            END
            ELSE BEGIN /* CLIENTE*/
               SELECT @V_COUNT = COUNT(*)
                 FROM CLIENTE
                WHERE CDCLIENTE = @C_CODCOME_CODIGO;
            END;

            IF @V_COUNT = 0 BEGIN
               INSERT INTO ERROCTBTEMP
                    (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
               VALUES
                   (@P_CDEMPRESA, @P_NRLANCCTBTEMP,@C_CODCOME_NRSEQCTBTEMP , '005');
            END;
            FETCH NEXT FROM C_CODCOME INTO @C_CODCOME_ID,@C_CODCOME_CODIGO,@C_CODCOME_NRSEQCTBTEMP;
         END;
         CLOSE C_CODCOME;
         DEALLOCATE C_CODCOME;
   END;


    /*VERIFICAO SE EXISTE ALGUMA PARTIDA ZERADA*/
    DECLARE C_PART CURSOR FOR
       (SELECT NRSEQCTBTEMP
         FROM LANCCTBTEMP
        WHERE CDEMPRESA = @P_CDEMPRESA
          AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
          AND VRLANCCTBTEMP = 0);

       OPEN C_PART;
       FETCH NEXT FROM C_PART INTO @C_PART_NRSEQCTBTEMP;
       WHILE (@@FETCH_STATUS = 0) BEGIN
           INSERT INTO ERROCTBTEMP
              (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
           VALUES
              (@P_CDEMPRESA, @P_NRLANCCTBTEMP,@C_PART_NRSEQCTBTEMP , '006');
           FETCH NEXT FROM C_PART INTO @C_PART_NRSEQCTBTEMP;
       END;
       CLOSE C_PART;
       DEALLOCATE C_PART;


    /*VERIFICAO SE EXISTE ALGUM RATEIO ZERADO*/
    IF @V_CONTRAT = 'S' BEGIN
       DECLARE C_RAT CURSOR FOR
          (SELECT NRSEQCTBTEMP
             FROM RATECTBTEMP
            WHERE CDEMPRESA = @P_CDEMPRESA
              AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
              AND VRRATECTBTEMP = 0);
          OPEN C_RAT;
          FETCH NEXT FROM C_RAT INTO @C_RAT_NRSEQCTBTEMP;
          WHILE (@@FETCH_STATUS = 0) BEGIN
             INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
               (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_RAT_NRSEQCTBTEMP , '007');
             FETCH NEXT FROM C_RAT INTO @C_RAT_NRSEQCTBTEMP;
          END;
          CLOSE C_RAT;
          DEALLOCATE C_RAT;
    END;


    /*VERIFICAO SE EXISTE ALGUMA FILIAL NO CADASTRADA*/
    IF @V_CONTRAT = 'S' BEGIN
       DECLARE C_FIL CURSOR FOR
          (SELECT CDFILIAL, NRSEQCTBTEMP
             FROM RATECTBTEMP
            WHERE CDEMPRESA = @P_CDEMPRESA
              AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP);

           OPEN C_FIL;
           FETCH NEXT FROM C_FIL INTO @C_FIL_CDFILIAL,@C_FIL_NRSEQCTBTEMP;
           WHILE (@@FETCH_STATUS = 0) BEGIN
              SELECT @V_COUNT = COUNT(*)
                FROM FILIAL
               WHERE CDFILIAL =  @C_FIL_CDFILIAL;

              IF @V_COUNT = 0 BEGIN
                 INSERT INTO ERROCTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
                 VALUES
                     (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_FIL_NRSEQCTBTEMP,'008');
              END; 
              FETCH NEXT FROM C_FIL INTO @C_FIL_CDFILIAL,@C_FIL_NRSEQCTBTEMP;
           END;
           CLOSE C_FIL;
           DEALLOCATE C_FIL;
    END;


    SET @V_RAT = 'S';
    IF @V_CONTRAT = 'S' BEGIN
    /*VERIFICAO SE EXISTE ALGUMA FILIAL NO INFORMADA*/
       DECLARE C_FIL_INF CURSOR FOR
          (SELECT NRSEQCTBTEMP
              FROM LANCCTBTEMP L, CONTCTBL C
             WHERE L.CDEMPRESA = @P_CDEMPRESA
               AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
               AND L.CDCONTCTBL = C.CDCONTCTBL
               AND C.IDCONTCCUS NOT IN ('L', 'O')
               AND NOT EXISTS (SELECT 1
                                 FROM RATECTBTEMP R
                                WHERE R.NRLANCCTBTEMP = L.NRLANCCTBTEMP
                                  AND R.NRSEQCTBTEMP = L.NRSEQCTBTEMP));

               OPEN C_FIL_INF;
               FETCH NEXT FROM C_FIL_INF INTO @C_FIL_INF_NRSEQCTBTEMP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SET @V_RAT = 'N';
                  INSERT INTO ERROCTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
                  VALUES
                     (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_FIL_INF_NRSEQCTBTEMP , '012');
                  FETCH NEXT FROM C_FIL_INF INTO @C_FIL_INF_NRSEQCTBTEMP;
               END;
               CLOSE C_FIL_INF;
               DEALLOCATE C_FIL_INF;
    END;
    IF @V_CONTRAT = 'S' BEGIN
       IF @V_RAT = 'S' BEGIN
         /*VERIFICAO SE EXISTE ALGUM CENTRO DE CUSTO NO CADASTRADO PARA FILIAL INFORMADA*/
          DECLARE C_CCUST CURSOR FOR
             ( SELECT CDFILIAL, CDCENTCUST, NRSEQCTBTEMP
                 FROM RATECTBTEMP
                WHERE CDEMPRESA = @P_CDEMPRESA
                  AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
                  AND CDCENTCUST IS NOT NULL
                  AND CDFILIAL IS NOT NULL
             );
             OPEN C_CCUST;
             FETCH NEXT FROM C_CCUST INTO @C_CCUST_CDFILIAL, @C_CCUST_CDCENTCUST, @C_CCUST_NRSEQCTBTEMP;
             WHILE (@@FETCH_STATUS = 0) BEGIN
                SELECT @V_COUNT = COUNT(*)
                  FROM CENTCUSTFIL
                 WHERE CDFILIAL = @C_CCUST_CDFILIAL
                   AND CDCENTCUST = @C_CCUST_CDCENTCUST;

                IF @V_COUNT = 0 BEGIN
                   INSERT INTO ERROCTBTEMP
                      (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
                   VALUES
                      (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_CCUST_NRSEQCTBTEMP , '009');
                END;
                FETCH NEXT FROM C_CCUST INTO @C_CCUST_CDFILIAL, @C_CCUST_CDCENTCUST, @C_CCUST_NRSEQCTBTEMP;
            END;
            CLOSE C_CCUST;
            DEALLOCATE C_CCUST;

            /* CENTRO DE CUSTO NO INFORMADO */
            DECLARE C_CCUST_INF CURSOR FOR
               ( SELECT NRSEQCTBTEMP
                   FROM RATECTBTEMP
                 WHERE CDEMPRESA = @P_CDEMPRESA
                    AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
                    AND CDCENTCUST IS NULL
                    AND CDFILIAL IS NOT NULL
                );
               OPEN C_CCUST_INF;
               FETCH NEXT FROM C_CCUST_INF INTO  @C_CCUST_INF_NRSEQCTBTEMP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  INSERT INTO  ERROCTBTEMP
                     (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
                  VALUES
                      (@P_CDEMPRESA, @P_NRLANCCTBTEMP,@C_CCUST_INF_NRSEQCTBTEMP,'013');
                  FETCH NEXT FROM C_CCUST_INF INTO  @C_CCUST_INF_NRSEQCTBTEMP;
               END;
               CLOSE C_CCUST_INF;
               DEALLOCATE C_CCUST_INF;
       END;
    END;

    /*VERIFICAO SE EXISTE SOMA DE RATEIOS  DIFERENTE DO VALOR DA PARTIDA*/
    IF @V_CONTRAT = 'S' BEGIN
       DECLARE C_DIF CURSOR FOR
       ( SELECT NRSEQCTBTEMP,VRLANCCTBTEMP
          FROM LANCCTBTEMP
         WHERE CDEMPRESA = @P_CDEMPRESA
           AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
       );
       OPEN C_DIF;
       FETCH NEXT FROM C_DIF INTO @C_DIF_NRSEQCTBTEMP, @C_DIF_VRLANCCTBTEMP;
       WHILE (@@FETCH_STATUS = 0) BEGIN
          SELECT @V_COUNT = SUM(VRRATECTBTEMP)
            FROM RATECTBTEMP
           WHERE CDEMPRESA     = @P_CDEMPRESA
             AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
             AND NRSEQCTBTEMP  = @C_DIF_NRSEQCTBTEMP;

          IF @V_COUNT <> @C_DIF_VRLANCCTBTEMP BEGIN
             INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_DIF_NRSEQCTBTEMP , '010');
          END;
          FETCH NEXT FROM C_DIF INTO @C_DIF_NRSEQCTBTEMP, @C_DIF_VRLANCCTBTEMP;
       END;
       CLOSE C_DIF;
       DEALLOCATE C_DIF;
    END;

    /*VERIFICA SE EXISTE ALGUMA PARTIDA DE CONTA COMERCIAL SEM CDIGO COMERCIAL*/
    IF @V_CONTCOME = 'S' BEGIN
       DECLARE C_CODCOME_NULL CURSOR FOR
       ( SELECT NRSEQCTBTEMP
           FROM LANCCTBTEMP
          WHERE CDEMPRESA = @P_CDEMPRESA
            AND NRLANCCTBTEMP = @P_NRLANCCTBTEMP
            AND IDLANCCTBCOME <> 'N'
            AND CDLANCCTBCOME IS NULL
       );
       OPEN C_CODCOME_NULL
       FETCH NEXT FROM C_CODCOME_NULL INTO @C_CODCOME_NULL_NRSEQCTBTEMP;
       WHILE (@@FETCH_STATUS = 0) BEGIN
          INSERT INTO ERROCTBTEMP
             (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
          VALUES
             (@P_CDEMPRESA, @P_NRLANCCTBTEMP, @C_CODCOME_NULL_NRSEQCTBTEMP,'011');
          FETCH NEXT FROM C_CODCOME_NULL INTO @C_CODCOME_NULL_NRSEQCTBTEMP;
       END;
       CLOSE C_CODCOME_NULL;
       DEALLOCATE C_CODCOME_NULL;
    END;

      /* VERIFICAO SE O LANAMENTO EST DENTRO DOS PERODOS ESPECFICOS PARA CADA MDULO */
   SELECT @V_IDSISTEMA = IDSISTEMA
     FROM LANCCTBTEMP
    WHERE NRLANCCTBTEMP = @P_NRLANCCTBTEMP
      AND CDEMPRESA = @P_CDEMPRESA;

   IF @V_IDSISTEMA IS NOT NULL BEGIN
      IF @V_IDSISTEMA IN ('BCO', 'PAG', 'REC') BEGIN
         SELECT @V_COUNT = COUNT(*)
           FROM CONTOPECTBL C, LANCCTBTEMP L
          WHERE C.CDEMPRESA = L.CDEMPRESA
            AND L.CDEMPRESA = @P_CDEMPRESA
            AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
            AND L.DTLANCCTBTEMP BETWEEN C.DTINICCTBL AND C.DTFINACCTBL;
         IF @V_COUNT = 0 BEGIN
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '015');
         END;
      END
      ELSE IF @V_IDSISTEMA = 'DFE' BEGIN
         SELECT @V_COUNT = COUNT(*)
           FROM CONTOPECTBL C, LANCCTBTEMP L
          WHERE C.CDEMPRESA = L.CDEMPRESA
            AND L.CDEMPRESA = @P_CDEMPRESA
            AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
            AND L.DTLANCCTBTEMP BETWEEN C.DTINIINTCONTDFE AND C.DTFININTCONTDFE;
         IF @V_COUNT = 0 BEGIN
            INSERT INTO ERROCTBTEMP
               (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
            VALUES
               (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '016');
         END;
      END
      ELSE IF @V_IDSISTEMA = 'DFS' BEGIN
         SELECT @V_COUNT = COUNT(*)
           FROM CONTOPECTBL C, LANCCTBTEMP L
          WHERE C.CDEMPRESA = L.CDEMPRESA
            AND L.CDEMPRESA = @P_CDEMPRESA
            AND L.NRLANCCTBTEMP = @P_NRLANCCTBTEMP
            AND L.DTLANCCTBTEMP BETWEEN C.DTINIINTCONTDFS AND C.DTFININTCONTDFS;
         IF @V_COUNT = 0 BEGIN
            INSERT INTO ERROCTBTEMP
                (CDEMPRESA, NRLANCCTBTEMP, NRSEQCTBTEMP, CDERROTEMP)
             VALUES
                (@P_CDEMPRESA, @P_NRLANCCTBTEMP, '', '017');
         END;
      END;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'VERIFICA_ERRO_CTB','5.50.000';
GO

